<?php

/**
 * @file
 * Admin page callback file for the guild module.
 */

/**
 * Form builder; The guild information form.
 *
 * @ingroup forms
 * @see system_settings_form()
 */
function wow_guild_admin_default($form, &$form_state) {
  module_load_include('inc', 'wow_realm');
  module_load_include('inc', 'wow', 'wow.admin');

  $realm = new stdClass();
  $realm->name = wow_guild_default('realm');
  $realm->region = wow_guild_default('region');

  $form['guild_information'] = array(
    '#type' => 'fieldset',
    '#title' => t('Guild details'),
  );
  $form['guild_information']['guild_realm'] = array(
    '#type' => 'wow_realm',
    '#title' => t('Realm'),
    '#default_value' => wow_realm_format($realm),
  );
  $form['actions']['#type'] = 'actions';

  if (($guild = variable_get('wow_guild_default')) != NULL) {
    $form['guild_information']['guild_realm']['#disabled'] = TRUE;
    $form['guild_information']['guild_name'] = array(
      '#type' => 'value',
      '#default_value' => $guild->name,
    );
    $form['guild_information']['guild'] = array(
      '#theme' => 'wow_admin_guild',
      '#guild' => wow_guild_load($guild->gid),
    );
    $form['guild_information']['site_default'] = array(
      '#type' => 'value',
      '#value' => $guild,
    );

    $form['guild_information']['#description'] = '<em>' . t('To delete guild data, you must disable and uninstall the WoW Guild module. This operation will erase all your data.') . '</em>';

    $form['actions']['refresh'] = array('#type' => 'submit', '#value' => t('Refresh'));

    $form['#submit'][] = 'wow_guild_admin_default_refresh';

    return $form;
  }
  else {
    $form['guild_information']['guild_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Guild name'),
      '#maxlength' => 24,
      '#size' => 30,
      '#default_value' => '',
      '#required' => TRUE,
    );

    $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));

    return $form;
  }
}

/**
 * Validates the submitted guild-information form.
 */
function wow_guild_admin_default_validate($form, &$form_state) {
  $realm = $form_state['values']['guild_realm'];
  $guild_name = check_plain($form_state['values']['guild_name']);

  if ($form_state['submitted'] && $guild_name) {
    try {
      $form_state['temporary']['guild'] = wow_guild_fetch($realm->region, $realm->slug, $guild_name, array('members', 'achievements'));
    }
    catch (WoWException $exception) {
      if ($exception->getCode() == 404) {
        $host = wow_api_info(variable_get('wow_region_default'), 'host');
        $url_guild = url("http://$host/wow/guild/$realm/$guild_name/");
        $args = array('%guild_name' => $guild_name, '%realm' => $realm, '!url_guild' => "<a href='$url_guild'>$url_guild</a>");
        form_set_error('guild_name', t('The guild name %guild_name does not exist on %realm. You should first check that your guild exists at: !url_guild.', $args));
      }
      else {
        watchdog_exception('wow_guild', $exception);
        form_set_error('guild_information', t('The website encountered an unexpected error. Please try again later.'));
      }
    }
  }
}

/**
 * Submits the validated guild-information form.
 */
function wow_guild_admin_default_submit($form, &$form_state) {
  $guild = $form_state['temporary']['guild'];

  $guild->gid = db_query('SELECT MAX(gid) FROM {wow_guild}')->fetchField() + 1;
  $guild->is_new = TRUE;

  // Set as site default.
  $guild_default = new stdClass();
  $guild_default->region = $guild->region;
  $guild_default->gid = $guild->gid;
  $guild_default->name = $guild->name;
  $guild_default->realm = $guild->realm;
  $guild_default->slug = $guild->slug;

  variable_set('wow_guild', $guild_default);

  // Insert a new guild record.
  $guild->save();
}

/**
 * Refresh the guild-information form by fetching the guilde from service.
 */
function wow_guild_admin_default_refresh($form, &$form_state) {
  $guild = $form_state['temporary']['guild'];
  $guild->gid = variable_get('wow_guild')->gid;

  try {
    $guild->save();
    drupal_set_message(t('Guild data has been updated.'));
  }
  catch (WoWException $exception) {
    if ($exception->getCode() == 304) {
      drupal_set_message(t("Service returned a '304 Not Modified' status meaning guild data are already up to date. If you want to refresh it anyway, try to force refresh (TO DO)."));
    }
    else {
      watchdog_exception('wow_guild', $exception);
      drupal_set_message(t('The website encountered an unexpected error. Please try again later.'));
    }
  }
}

/**
 * Selects just the realm dropdown to be returned for re-rendering.
 *
 * Since the controlling logic for populating the form is in the form builder
 * function, all we do here is select the element and return it to be updated.
 *
 * @return array
 *   The realm dropdown.
 */
function wow_guild_information_region_callback($form, $form_state) {
  return $form['guild_information']['guild_realm'];
}

/**
 * Returns the HTML for a guild information.
 *
 * @param array $variables
 *   An associative array containing:
 *   - guild: A render element representing a guild.
 *
 * @ingroup themeable
 */
function theme_wow_admin_guild($variables) {
  $guild = $variables['guild']['#guild'];

  $render = '<div class="name"><strong>' . t('Guild: @name', array('@name' => $guild->name)) . '</strong></div>';
  $render .= '<div class="side">' . t('Side: !side', array('!side' => wow_side($guild->side ? 'horde' : 'alliance'))) . '</div>';
  $render .= '<div class="level">' . t('Level: @level', array('@level' => $guild->level)) . '</div>';
  $render .= '<div class="achievement">' . t('Achievement Points: @achievementPoints', array('@achievementPoints' => $guild->achievementPoints)) . '</div>';
  $render .= '<div class="last-modified"><em>' . t('Last modified: !lastModified', array('!lastModified' => format_date($guild->lastModified))) . '</em>';

  return '<div id="guild-' . $guild->gid . '" class="guild">' . $render . '</div>';
}
